﻿;{ ==Code Header Comment==============================
;        Name/title: VDraw.pbi
;   Executable name: Part of Icondesigner.exe
;           Version: 1.0.0
;            Author: Collectordave
;     Collaborators: 
;    Translation by: 
;       Create date: 16\05\2016
; Previous releases: 
; This Release Date: 
;  Operating system: Windows  [X]GUI
;  Compiler version: PureBasic 5.42LTS(x64)
;         Copyright: (C)2016
;           License: 
;         Libraries: 
;     English Forum: 
;      French Forum: 
;      German Forum: 
;  Tested platforms: Windows
; ====================================================
;.......10........20........30........40........50........60........70........80
;
   ;--------------------------------------------------------------------
   
   ; Replacement of AddPathArc() command, which is buggy at least in PB 5.42 on Windows and Linux
   ;
   ; [by Little John;
   ;  slightly modified after
   ;  <http://stackoverflow.com/questions/24771828/algorithm-for-creating-rounded-corners-in-a-polygon>, 2016-03-28]
   
;}

DeclareModule VDraw
   
   Declare.i Color_Darken (Color.i, Fact.d)
   Declare.i StartNewVectorImage (img.i, width.i, height.i, depth.i=24, backColor.i=0, unit.i=#PB_Unit_Pixel)
   Declare   AddPathRoundedCorner (x1.d, y1.d, x2.d, y2.d, radius.d, flags.i=#PB_Path_Default)
   
   Macro AddPathArc (_x1_, _y1_, _x2_, _y2_, _radius_, _flags_=#PB_Path_Default)
     VDraw::AddPathRoundedCorner(_x1_, _y1_, _x2_, _y2_, _radius_, _flags_)
   EndMacro
    
EndDeclareModule

Module VDraw
  EnableExplicit
      
  Procedure.i Color_Darken (Color.i, Fact.d)
    ; [by Oma, 2016-04-21]
    Protected.i Red, Green, Blue, Alpha
      
    If Fact > 1.0 
      Fact = 1.0
    EndIf
    Alpha = Alpha(Color)
    Red   = Red(Color)   * Fact
    Green = Green(Color) * Fact
    Blue  = Blue(Color)  * Fact
      
    ProcedureReturn RGBA(Red, Green, Blue, Alpha)
    
  EndProcedure
    
  Procedure.i StartNewVectorImage (img.i, width.i, height.i, depth.i=24, backColor.i=0, unit.i=#PB_Unit_Pixel)

      Protected ret.i
      
      ret = CreateImage(img, width, height, depth, backColor)
      If ret
         If img = #PB_Any
            img = ret
         EndIf
         If StartVectorDrawing(ImageVectorOutput(img, unit)) = 0
            FreeImage(img)
            ret = 0
         EndIf
      EndIf
      
      ProcedureReturn ret
      
   EndProcedure
   

   Macro GetLength (_a_, _b_)
      Sqr((_a_) * (_a_) + (_b_) * (_b_))
   EndMacro
   
   Macro GetProportionPointX (_px_, _segment_, _length_, _dx_)
      ((_px_) - (_dx_) * (_segment_) / (_length_))
   EndMacro
   
   Macro GetProportionPointY (_py_, _segment_, _length_, _dy_)
      ((_py_) - (_dy_) * (_segment_) / (_length_))
   EndMacro
      
   Procedure AddPathRoundedCorner (x1.d, y1.d, x2.d, y2.d, radius.d, flags.i=#PB_Path_Default)
      ; Meaning of the parameters: same as for the built-in command AddPathArc()
      Protected.d dx1, dy1, dx2, dy2, angle, tan, segment, length1, length2, length, c2x, c2y, dx, dy, L, po
      Protected.d c1x, c1y, circlePointX, circlePointY, startangle, endangle, sweepAngle
      Protected.i circleFlags
      
      If flags & #PB_Path_Relative
         x1 + PathCursorX()
         y1 + PathCursorY()
         x2 + PathCursorX()
         y2 + PathCursorY()
      EndIf
      
      ; Vector 1
      dx1 = x1 - PathCursorX()
      dy1 = y1 - PathCursorY()
      
      ; Vector 2
      dx2 = x1 - x2
      dy2 = y1 - y2
      
      ; Angle between vector 1 and vector 2 divided by 2
      angle = (ATan2(dx1, dy1) - ATan2(dx2, dy2)) / 2
      
      ; The length of segment between the angular point and the
      ; points of intersection with the circle of a given radius
      tan = Abs(Tan(angle))
      segment = radius / tan
      
      ; Check the segment
      length1 = GetLength(dx1, dy1)
      length2 = GetLength(dx2, dy2)
      length = length1
      If length > length2
         length = length2
      EndIf
      
      If segment > length
         segment = length
         radius = length * tan
      EndIf
      
      ; Points of intersection are calculated by the proportion between
      ; the coordinates of the vector, length of vector and the length of the segment.
      c1x = GetProportionPointX(x1, segment, length1, dx1)
      c1y = GetProportionPointY(y1, segment, length1, dy1)
      c2x = GetProportionPointX(x1, segment, length2, dx2)
      c2y = GetProportionPointY(y1, segment, length2, dy2)
      
      ; Calculation of the coordinates of the circle center
      ; by the addition of angular vectors
      dx = x1 * 2 - c1x - c2x
      dy = y1 * 2 - c1y - c2y
      
      L = GetLength(dx, dy)
      po = GetLength(segment, radius)
      
      circlePointX = GetProportionPointX(x1, po, L, dx)
      circlePointY = GetProportionPointY(y1, po, L, dy)
      
      ; StartAngle and EndAngle of arc
      startangle = ATan2(c1x - circlePointX, c1y - circlePointY)
      endangle   = ATan2(c2x - circlePointX, c2y - circlePointY)
      
      ; Additional checks
      sweepAngle = endAngle - startAngle
      circleFlags = #PB_Path_Connected
      If (-#PI < sweepAngle And sweepAngle < 0) Or sweepAngle > #PI
         circleFlags | #PB_Path_CounterClockwise
      EndIf
      
      ; Draw result
      AddPathLine(c1x, c1y)
      AddPathCircle(circlePointX, circlePointY, radius, Degree(startAngle), Degree(endAngle), circleFlags)

   EndProcedure
EndModule
; IDE Options = PureBasic 5.42 LTS (Windows - x64)
; CursorPosition = 19
; FirstLine = 6
; Folding = -w
; EnableUnicode
; EnableXP